@@ -1,4 +1,4 @@
-/*	$NetBSD: job.c,v 1.21 1997/09/28 03:31:05 lukem Exp $	*/
+/*	$NetBSD: job.c,v 1.25 1999/02/04 00:40:55 cjs Exp $	*/
 
 /*
  * Copyright (c) 1988, 1989, 1990 The Regents of the University of California.
@@ -39,14 +39,14 @@
  */
 
 #ifdef MAKE_BOOTSTRAP
-static char rcsid[] = "$NetBSD: job.c,v 1.21 1997/09/28 03:31:05 lukem Exp $";
+static char rcsid[] = "$NetBSD: job.c,v 1.25 1999/02/04 00:40:55 cjs Exp $";
 #else
 #include <sys/cdefs.h>
 #ifndef lint
 #if 0
 static char sccsid[] = "@(#)job.c	8.2 (Berkeley) 3/19/94";
 #else
-__RCSID("$NetBSD: job.c,v 1.21 1997/09/28 03:31:05 lukem Exp $");
+__RCSID("$NetBSD: job.c,v 1.25 1999/02/04 00:40:55 cjs Exp $");
 #endif
 #endif /* not lint */
 #endif
@@ -79,6 +79,8 @@ __RCSID("$NetBSD: job.c,v 1.21 1997/09/28 03:31:05 lukem Exp $");
  *	    	  	    	Hence, the makefile must have been parsed
  *	    	  	    	before this function is called.
  *
+ *	Job_End  	    	Cleanup any memory used.
+ *
  *	Job_Full  	    	Return TRUE if the job table is filled.
  *
  *	Job_Empty 	    	Return TRUE if the job table is completely
@@ -88,7 +90,7 @@ __RCSID("$NetBSD: job.c,v 1.21 1997/09/28 03:31:05 lukem Exp $");
  *	    	  	    	the line as a shell specification. Returns
  *	    	  	    	FAILURE if the spec was incorrect.
  *
- *	Job_End	  	    	Perform any final processing which needs doing.
+ *	Job_Finish	    	Perform any final processing which needs doing.
  *	    	  	    	This includes the execution of any commands
  *	    	  	    	which have been/were attached to the .END
  *	    	  	    	target. It should only be called when the
@@ -167,12 +169,10 @@ static int     	  numCommands; 	    /* The number of commands actually printed
 #define JOB_STOPPED	3   	/* The job is stopped */
 
 /*
- * tfile is the name of a file into which all shell commands are put. It is
- * used over by removing it before the child shell is executed. The XXXXX in
- * the string are replaced by the pid of the make process in a 5-character
- * field with leading zeroes.
+ * tfile is used to build temp file names to store shell commands to
+ * execute.
  */
-static char     tfile[] = TMPPAT;
+static char     tfile[sizeof(TMPPAT)];
 
 
 /*
@@ -201,7 +201,7 @@ static Shell    shells[] = {
 #ifdef OLDBOURNESHELL
     FALSE, "echo \"%s\"\n", "sh -c '%s || exit 0'\n",
 #endif
-    "v", "e",
+    "vq", "e",
 },
     /*
      * UNKNOWN.
@@ -220,7 +220,8 @@ static Shell 	*commandShell = &shells[DEFSHELL];/* this is the shell to
 						   * Job_ParseShell function */
 static char   	*shellPath = NULL,		  /* full pathname of
 						   * executable image */
-               	*shellName;	      	      	  /* last component of shell */
+               	*shellName = NULL,	      	  /* last component of shell */
+		*shellArgv = NULL;		  /* Custom shell args */
 
 
 static int  	maxJobs;    	/* The most children we can run at once */
@@ -640,6 +641,7 @@ JobPrintCommand(cmdp, jobp)
     }
 
     DBPRINTF(cmdTemplate, cmd);
+    free(cmdStart);
 
     if (errOff) {
 	/*
@@ -994,13 +996,11 @@ JobFinish(job, status)
 	aborting = ABORT_ERROR;
     }
 
-    if ((aborting == ABORT_ERROR) && Job_Empty()) {
+    if ((aborting == ABORT_ERROR) && Job_Empty())
 	/*
 	 * If we are aborting and the job table is now empty, we finish.
 	 */
-	(void) eunlink(tfile);
 	Finish(errors);
-    }
 }
 
 /*-
@@ -1667,10 +1667,10 @@ JobStart(gn, flags, previous)
 {
     register Job  *job;       /* new job descriptor */
     char	  *argv[4];   /* Argument vector to shell */
-    static int    jobno = 0;  /* job number of catching output in a file */
     Boolean	  cmdsOK;     /* true if the nodes commands were all right */
     Boolean 	  local;      /* Set true if the job was run locally */
     Boolean 	  noExec;     /* Set true if we decide not to run the job */
+    int           tfd;        /* where to stash those pesky commands */
 
     if (previous != NULL) {
 	previous->flags &= ~(JOB_FIRST|JOB_IGNERR|JOB_SILENT|JOB_REMOTE);
@@ -1725,9 +1725,14 @@ JobStart(gn, flags, previous)
 	    DieHorribly();
 	}
 
-	job->cmdFILE = fopen(tfile, "w+");
+        (void) strcpy(tfile, TMPPAT);
+        if ((tfd = mkstemp(tfile)) == -1)
+            Punt("Cannot create temp file: %s", strerror(errno));
+        job->cmdFILE = fdopen(tfd, "w+");
+        eunlink(tfile);
 	if (job->cmdFILE == NULL) {
 	    Punt("Could not open %s", tfile);
+	    close(tfd);
 	}
 	(void) fcntl(FILENO(job->cmdFILE), F_SETFD, 1);
 	/*
@@ -1833,7 +1838,6 @@ JobStart(gn, flags, previous)
 	 * Unlink and close the command file if we opened one
 	 */
 	if (job->cmdFILE != stdout) {
-	    (void) eunlink(tfile);
 	    if (job->cmdFILE != NULL)
 		(void) fclose(job->cmdFILE);
 	} else {
@@ -1861,7 +1865,6 @@ JobStart(gn, flags, previous)
 	}
     } else {
 	(void) fflush(job->cmdFILE);
-	(void) eunlink(tfile);
     }
 
     /*
@@ -1873,8 +1876,7 @@ JobStart(gn, flags, previous)
     /*
      * If we're using pipes to catch output, create the pipe by which we'll
      * get the shell's output. If we're using files, print out that we're
-     * starting a job and then set up its temporary-file name. This is just
-     * tfile with two extra digits tacked on -- jobno.
+     * starting a job and then set up its temporary-file name.
      */
     if (!compatMake || (job->flags & JOB_FIRST)) {
 	if (usePipes) {
@@ -1888,9 +1890,9 @@ JobStart(gn, flags, previous)
 	} else {
 	    (void) fprintf(stdout, "Remaking `%s'\n", gn->name);
   	    (void) fflush(stdout);
-	    sprintf(job->outFile, "%s%02d", tfile, jobno);
-	    jobno = (jobno + 1) % 100;
-	    job->outFd = open(job->outFile,O_WRONLY|O_CREAT|O_APPEND,0600);
+            (void) strcpy(job->outFile, TMPPAT);
+            if ((job->outFd = mkstemp(job->outFile)) == -1)
+               Punt("Cannot create temp file: %s", strerror(errno));
 	    (void) fcntl(job->outFd, F_SETFD, 1);
 	}
     }
@@ -2407,8 +2409,6 @@ Job_Init(maxproc, maxlocal)
 {
     GNode         *begin;     /* node for commands to do at the very start */
 
-    (void) sprintf(tfile, "/tmp/make%05d", getpid());
-
     jobs =  	  Lst_Init(FALSE);
     stoppedJobs = Lst_Init(FALSE);
     maxJobs = 	  maxproc;
@@ -2659,50 +2659,55 @@ Job_ParseShell(line)
     Shell   	  newShell;
     Boolean 	  fullSpec = FALSE;
 
-    while (isspace(*line)) {
+    while (isspace((unsigned char)*line)) {
 	line++;
     }
-    words = brk_string(line, &wordCount, TRUE);
+
+    if (shellArgv)
+	free(shellArgv);
+
+    words = brk_string(line, &wordCount, TRUE, &shellArgv);
 
     memset((Address)&newShell, 0, sizeof(newShell));
 
     /*
      * Parse the specification by keyword
      */
-    for (path = NULL, argc = wordCount - 1, argv = words + 1;
-	 argc != 0;
-	 argc--, argv++) {
-	     if (strncmp(*argv, "path=", 5) == 0) {
-		 path = &argv[0][5];
-	     } else if (strncmp(*argv, "name=", 5) == 0) {
-		 newShell.name = &argv[0][5];
-	     } else {
-		 if (strncmp(*argv, "quiet=", 6) == 0) {
-		     newShell.echoOff = &argv[0][6];
-		 } else if (strncmp(*argv, "echo=", 5) == 0) {
-		     newShell.echoOn = &argv[0][5];
-		 } else if (strncmp(*argv, "filter=", 7) == 0) {
-		     newShell.noPrint = &argv[0][7];
-		     newShell.noPLen = strlen(newShell.noPrint);
-		 } else if (strncmp(*argv, "echoFlag=", 9) == 0) {
-		     newShell.echo = &argv[0][9];
-		 } else if (strncmp(*argv, "errFlag=", 8) == 0) {
-		     newShell.exit = &argv[0][8];
-		 } else if (strncmp(*argv, "hasErrCtl=", 10) == 0) {
-		     char c = argv[0][10];
-		     newShell.hasErrCtl = !((c != 'Y') && (c != 'y') &&
+    for (path = NULL, argc = wordCount - 1, argv = words;
+	argc != 0;
+	argc--, argv++) {
+	    if (strncmp(*argv, "path=", 5) == 0) {
+		path = &argv[0][5];
+	    } else if (strncmp(*argv, "name=", 5) == 0) {
+		newShell.name = &argv[0][5];
+	    } else {
+		if (strncmp(*argv, "quiet=", 6) == 0) {
+		    newShell.echoOff = &argv[0][6];
+		} else if (strncmp(*argv, "echo=", 5) == 0) {
+		    newShell.echoOn = &argv[0][5];
+		} else if (strncmp(*argv, "filter=", 7) == 0) {
+		    newShell.noPrint = &argv[0][7];
+		    newShell.noPLen = strlen(newShell.noPrint);
+		} else if (strncmp(*argv, "echoFlag=", 9) == 0) {
+		    newShell.echo = &argv[0][9];
+		} else if (strncmp(*argv, "errFlag=", 8) == 0) {
+		    newShell.exit = &argv[0][8];
+		} else if (strncmp(*argv, "hasErrCtl=", 10) == 0) {
+		    char c = argv[0][10];
+		    newShell.hasErrCtl = !((c != 'Y') && (c != 'y') &&
 					   (c != 'T') && (c != 't'));
-		 } else if (strncmp(*argv, "check=", 6) == 0) {
-		     newShell.errCheck = &argv[0][6];
-		 } else if (strncmp(*argv, "ignore=", 7) == 0) {
-		     newShell.ignErr = &argv[0][7];
-		 } else {
-		     Parse_Error(PARSE_FATAL, "Unknown keyword \"%s\"",
-				  *argv);
-		     return(FAILURE);
-		 }
-		 fullSpec = TRUE;
-	     }
+		} else if (strncmp(*argv, "check=", 6) == 0) {
+		    newShell.errCheck = &argv[0][6];
+		} else if (strncmp(*argv, "ignore=", 7) == 0) {
+		    newShell.ignErr = &argv[0][7];
+		} else {
+		    Parse_Error(PARSE_FATAL, "Unknown keyword \"%s\"",
+				*argv);
+		    free(words);
+		    return(FAILURE);
+		}
+		fullSpec = TRUE;
+	    }
     }
 
     if (path == NULL) {
@@ -2762,7 +2767,7 @@ Job_ParseShell(line)
 
     /*
      * Do not free up the words themselves, since they might be in use by the
-     * shell specification...
+     * shell specification.
      */
     free(words);
     return SUCCESS;
@@ -2912,26 +2917,21 @@ JobInterrupt(runINTERRUPT, signo)
 	    }
 	}
     }
-    (void) eunlink(tfile);
     exit(signo);
 }
 
 /*
  *-----------------------------------------------------------------------
- * Job_End --
+ * Job_Finish --
  *	Do final processing such as the running of the commands
  *	attached to the .END target.
  *
  * Results:
  *	Number of errors reported.
- *
- * Side Effects:
- *	The process' temporary file (tfile) is removed if it still
- *	existed.
  *-----------------------------------------------------------------------
  */
 int
-Job_End()
+Job_Finish()
 {
     if (postCommands != NILGNODE && !Lst_IsEmpty(postCommands->commands)) {
 	if (errors) {
@@ -2947,10 +2947,28 @@ Job_End()
 	    }
 	}
     }
-    (void) eunlink(tfile);
     return(errors);
 }
 
+/*-
+ *-----------------------------------------------------------------------
+ * Job_End --
+ *	Cleanup any memory used by the jobs module
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Memory is freed
+ *-----------------------------------------------------------------------
+ */
+void
+Job_End()
+{
+    if (shellArgv)
+	free(shellArgv);
+}
+
 /*-
  *-----------------------------------------------------------------------
  * Job_Wait --
@@ -3031,7 +3049,6 @@ Job_AbortAll()
      */
     while (waitpid((pid_t) -1, &foo, WNOHANG) > 0)
 	continue;
-    (void) eunlink(tfile);
 }
 
 #ifdef REMOTE
